from typing import Callable, Tuple, Type

from starlette import status
from starlette.requests import Request
from starlette.responses import Response

from throttled.exceptions import RateLimitExceeded

ExceptionType = Type[Exception]
HandlerType = Callable[[Request, Exception], Response]

FastAPIHandler = Tuple[ExceptionType, HandlerType]


def handle_rate_exceed_func(request: Request, exc: RateLimitExceeded) -> Response:
    return Response(
        status_code=status.HTTP_429_TOO_MANY_REQUESTS,
        content=f"Rate exceeded for key={exc.key}.",
        headers={"Retry-After": str(exc.retry_after)},
    )


handle_rate_exceed: FastAPIHandler = (RateLimitExceeded, handle_rate_exceed_func)



from abc import ABC
from typing import Callable, Sequence

from fastapi import Depends
from starlette.requests import Request

from throttled import Hit
from throttled.strategies import Strategy


class Limiter(ABC):
    def __init__(self, strategy: Strategy):
        self._strategy = strategy


class GlobalLimiter(Limiter):
    def __call__(self):
        hit = Hit()
        self._strategy(hit)


class HostBasedLimiter(Limiter):
    def __call__(self, request: Request):
        hit = Hit(key=request.client.host)
        self._strategy(hit)


def as_dependencies(limiters: Sequence[Callable]) -> Depends:
    return [Depends(limiter) for limiter in limiters]
